@@ -134,20 +134,14 @@ void buf_Release(cm_buf_t *bp)
     lock_ReleaseWrite(&buf_globalLock);
 }
 
-/* incremental sync daemon.  Writes 1/10th of all the buffers every 5000 ms */
+/* incremental sync daemon.  Writes all dirty buffers every 5000 ms */
 void buf_IncrSyncer(long parm)
 {
-    cm_buf_t *bp;			/* buffer we're hacking on; held */
+    cm_buf_t **bpp, *bp;
     long i;				/* counter */
-    long wasDirty;
+    long wasDirty = 0;
     cm_req_t req;
 
-    lock_ObtainWrite(&buf_globalLock);
-    bp = cm_data.buf_allp;
-    buf_HoldLocked(bp);
-    lock_ReleaseWrite(&buf_globalLock);
-    wasDirty = 0;
-
     while (buf_ShutdownFlag == 0) {
 	if (!wasDirty) {
 #ifndef DJGPP
@@ -158,36 +152,49 @@ void buf_IncrSyncer(long parm)
 #endif /* DJGPP */
 	}
 
-        if (buf_ShutdownFlag == 1)
-            return;
-
 	wasDirty = 0;
 
         /* now go through our percentage of the buffers */
-        for (i=0; i<cm_data.buf_nbuffers; i++) {
-            /* don't want its identity changing while we're
-             * messing with it, so must do all of this with
-             * bp held.
-             */
+        for (bpp = &cm_data.buf_dirtyListp; bp = *bpp; ) {
 
-            /* start cleaning the buffer; don't touch log pages since
-             * the log code counts on knowing exactly who is writing
-             * a log page at any given instant.
-             */
-            cm_InitReq(&req);
-            req.flags |= CM_REQ_NORETRY;
-	    wasDirty |= buf_CleanAsync(bp, &req);
+	    /* all dirty buffers are held when they are added to the
+	     * dirty list.  No need for an additional hold.
+	     */
 
-            /* now advance to the next buffer; the allp chain never changes,
-             * and so can be followed even when holding no locks.
-             */
-            lock_ObtainWrite(&buf_globalLock);
-            buf_ReleaseLocked(bp);
-            bp = bp->allp;
-            if (!bp) 
-                bp = cm_data.buf_allp;
-	    buf_HoldLocked(bp);
-            lock_ReleaseWrite(&buf_globalLock);
+	    if (bp->flags & CM_BUF_DIRTY) {
+		/* start cleaning the buffer; don't touch log pages since
+ 		 * the log code counts on knowing exactly who is writing
+		 * a log page at any given instant.
+		 */
+		cm_InitReq(&req);
+		req.flags |= CM_REQ_NORETRY;
+		wasDirty |= buf_CleanAsync(bp, &req);
+	    }
+
+	    /* the buffer may or may not have been dirty
+	     * and if dirty may or may not have been cleaned
+	     * successfully.  check the dirty flag again.  
+	     */
+	    if (!(bp->flags & CM_BUF_DIRTY)) {
+		lock_ObtainMutex(&bp->mx);
+		if (!(bp->flags & CM_BUF_DIRTY)) {
+		    /* remove the buffer from the dirty list */
+		    lock_ObtainWrite(&buf_globalLock);
+		    *bpp = bp->dirtyp;
+		    bp->dirtyp = NULL;
+		    if (cm_data.buf_dirtyListp == NULL)
+			cm_data.buf_dirtyListEndp = NULL;
+		    buf_ReleaseLocked(bp);
+		    lock_ReleaseWrite(&buf_globalLock);
+		} else {
+		    /* advance the pointer so we don't loop forever */
+		    bpp = &bp->dirtyp;
+		}
+		lock_ReleaseMutex(&bp->mx);
+	    } else {
+		/* advance the pointer so we don't loop forever */
+		bpp = &bp->dirtyp;
+	    }
         }	/* for loop over a bunch of buffers */
     }		/* whole daemon's while loop */
 }
@@ -306,10 +313,10 @@ long buf_Init(int newFile, cm_buf_ops_t *opsp, afs_uint64 nbuffers)
             cm_data.buf_nOrigBuffers = cm_data.buf_nbuffers;
  
             /* lower hash size to a prime number */
-            cm_data.buf_hashSize = osi_PrimeLessThan(CM_BUF_HASHSIZE);
+	    cm_data.buf_hashSize = osi_PrimeLessThan((afs_uint32)(cm_data.buf_nbuffers/7 + 1));
  
             /* create hash table */
-            memset((void *)cm_data.buf_hashTablepp, 0, cm_data.buf_hashSize * sizeof(cm_buf_t *));
+            memset((void *)cm_data.buf_scacheHashTablepp, 0, cm_data.buf_hashSize * sizeof(cm_buf_t *));
             
             /* another hash table */
             memset((void *)cm_data.buf_fileHashTablepp, 0, cm_data.buf_hashSize * sizeof(cm_buf_t *));
@@ -541,11 +548,11 @@ void buf_WaitIO(cm_scache_t * scp, cm_buf_t *bp)
  */
 cm_buf_t *buf_FindLocked(struct cm_scache *scp, osi_hyper_t *offsetp)
 {
-    long i;
+    afs_uint32 i;
     cm_buf_t *bp;
 
     i = BUF_HASH(&scp->fid, offsetp);
-    for(bp = cm_data.buf_hashTablepp[i]; bp; bp=bp->hashp) {
+    for(bp = cm_data.buf_scacheHashTablepp[i]; bp; bp=bp->hashp) {
         if (cm_FidCmp(&scp->fid, &bp->fid) == 0
              && offsetp->LowPart == bp->offset.LowPart
              && offsetp->HighPart == bp->offset.HighPart) {
@@ -586,6 +593,7 @@ long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
     long code = 0;
     long isdirty = 0;
     cm_scache_t * scp = NULL;
+    osi_hyper_t offset;
 
     osi_assert(bp->magic == CM_BUF_MAGIC);
 
@@ -596,9 +604,10 @@ long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
 	scp = cm_FindSCache(&bp->fid);
 	if (scp) {
 	    osi_Log2(buf_logp, "buf_CleanAsyncLocked starts I/O on scp 0x%p buf 0x%p", scp, bp);
-	    code = (*cm_buf_opsp->Writep)(scp, &bp->offset,
-					   cm_data.buf_blockSize, 0, bp->userp,
-					   reqp);
+
+            offset = bp->offset;
+            LargeIntegerAdd(offset, ConvertLongToLargeInteger(bp->dirty_offset));
+	    code = (*cm_buf_opsp->Writep)(scp, &offset, bp->dirty_length, 0, bp->userp, reqp);
 	    osi_Log3(buf_logp, "buf_CleanAsyncLocked I/O on scp 0x%p buf 0x%p, done=%d", scp, bp, code);
 
 	    cm_ReleaseSCache(scp);
@@ -616,6 +625,8 @@ long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
 	if (code == CM_ERROR_NOSUCHFILE){
 	    bp->flags &= ~CM_BUF_DIRTY;
 	    bp->flags |= CM_BUF_ERROR;
+            bp->dirty_offset = 0;
+            bp->dirty_length = 0;
 	    bp->error = CM_ERROR_NOSUCHFILE;
 	    bp->dataVersion = -1; /* bad */
 	    bp->dirtyCounter++;
@@ -635,6 +646,11 @@ long buf_CleanAsyncLocked(cm_buf_t *bp, cm_req_t *reqp)
 	    break;
     };
 
+    if (!(bp->flags & CM_BUF_DIRTY)) {
+	/* remove buffer from dirty buffer queue */
+
+    }
+
     /* do logging after call to GetLastError, or else */
         
     /* if someone was waiting for the I/O that just completed or failed,
@@ -668,8 +684,8 @@ void buf_Recycle(cm_buf_t *bp)
      * have any lock conflicts, so we can grab the buffer lock out of
      * order in the locking hierarchy.
      */
-    osi_Log2( buf_logp, "buf_Recycle recycles 0x%p, off 0x%x",
-              bp, bp->offset.LowPart);
+    osi_Log3( buf_logp, "buf_Recycle recycles 0x%p, off 0x%x:%08x",
+              bp, bp->offset.HighPart, bp->offset.LowPart);
 
     osi_assert(bp->refCount == 0);
     osi_assert(!(bp->flags & (CM_BUF_READING | CM_BUF_WRITING | CM_BUF_DIRTY)));
@@ -679,7 +695,7 @@ void buf_Recycle(cm_buf_t *bp)
         /* Remove from hash */
 
         i = BUF_HASH(&bp->fid, &bp->offset);
-        lbpp = &(cm_data.buf_hashTablepp[i]);
+        lbpp = &(cm_data.buf_scacheHashTablepp[i]);
         for(tbp = *lbpp; tbp; lbpp = &tbp->hashp, tbp = *lbpp) {
             if (tbp == bp) break;
         }
@@ -756,11 +772,12 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
 
 	/* does this fix the problem below?  it's a simple solution. */
 	if (!cm_data.buf_freeListEndp)
-	    {
+	{
 	    lock_ReleaseWrite(&buf_globalLock);
+	    osi_Log0(afsd_logp, "buf_GetNewLocked: Free Buffer List is empty - sleeping 200ms");
 	    Sleep(200);
 	    goto retry;
-	    }
+	}
 
         /* for debugging, assert free list isn't empty, although we
          * really should try waiting for a running tranasction to finish
@@ -851,8 +868,8 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
 #endif
                 bp->offset = *offsetp;
                 i = BUF_HASH(&scp->fid, offsetp);
-                bp->hashp = cm_data.buf_hashTablepp[i];
-                cm_data.buf_hashTablepp[i] = bp;
+                bp->hashp = cm_data.buf_scacheHashTablepp[i];
+                cm_data.buf_scacheHashTablepp[i] = bp;
                 i = BUF_FILEHASH(&scp->fid);
                 nextBp = cm_data.buf_fileHashTablepp[i];
                 bp->fileHashp = nextBp;
@@ -862,17 +879,12 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
                 cm_data.buf_fileHashTablepp[i] = bp;
             }
 
-            /* prepare to return it.  Start by giving it a good
-             * refcount */
-            bp->refCount = 1;
-                        
-            /* and since it has a non-zero ref count, we should move
-             * it from the lru queue.  It better be still there,
-             * since we've held the global (big) lock since we found
-             * it there.
+            /* we should move it from the lru queue.  It better still be there,
+             * since we've held the global (big) lock since we found it there.
              */
             osi_assertx(bp->flags & CM_BUF_INLRU,
                          "buf_GetNewLocked: LRU screwup");
+
             if (cm_data.buf_freeListEndp == bp) {
                 /* we're the last guy in this queue, so maintain it */
                 cm_data.buf_freeListEndp = (cm_buf_t *) osi_QPrev(&bp->q);
@@ -880,13 +892,19 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
             osi_QRemove((osi_queue_t **) &cm_data.buf_freeListp, &bp->q);
             bp->flags &= ~CM_BUF_INLRU;
 
-            /* finally, grab the mutex so that people don't use it
+            /* grab the mutex so that people don't use it
              * before the caller fills it with data.  Again, no one	
              * should have been able to get to this dude to lock it.
              */
-            osi_assertx(lock_TryMutex(&bp->mx),
-                         "buf_GetNewLocked: TryMutex failed");
+	    if (!lock_TryMutex(&bp->mx)) {
+	    	osi_Log2(afsd_logp, "buf_GetNewLocked bp 0x%p cannot be mutex locked.  refCount %d should be 0",
+			 bp, bp->refCount);
+		osi_panic("buf_GetNewLocked: TryMutex failed",__FILE__,__LINE__);
+	    }
 
+	    /* prepare to return it.  Give it a refcount */
+            bp->refCount = 1;
+                        
             lock_ReleaseWrite(&buf_globalLock);
             *bufpp = bp;
 
@@ -896,6 +914,8 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bu
             return 0;
         } /* for all buffers in lru queue */
         lock_ReleaseWrite(&buf_globalLock);
+	osi_Log0(afsd_logp, "buf_GetNewLocked: Free Buffer List has no buffers with a zero refcount - sleeping 100ms");
+	Sleep(100);		/* give some time for a buffer to be freed */
     }	/* while loop over everything */
     /* not reached */
 } /* the proc */
@@ -948,8 +968,8 @@ long buf_GetNew(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bufpp)
      */
     lock_ReleaseMutex(&bp->mx);
     *bufpp = bp;
-    osi_Log3(buf_logp, "buf_GetNew returning bp 0x%p for scp 0x%p, offset 0x%x",
-              bp, scp, offsetp->LowPart);
+    osi_Log4(buf_logp, "buf_GetNew returning bp 0x%p for scp 0x%p, offset 0x%x:%08x",
+              bp, scp, offsetp->HighPart, offsetp->LowPart);
     return 0;
 }
 
@@ -1018,12 +1038,6 @@ long buf_Get(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bufpp)
         /* load the page; freshly created pages should be idle */
         osi_assert(!(bp->flags & (CM_BUF_READING | CM_BUF_WRITING)));
 
-        /* setup offset, event */
-#ifndef DJGPP  /* doesn't seem to be used */
-        bp->over.Offset = bp->offset.LowPart;
-        bp->over.OffsetHigh = bp->offset.HighPart;
-#endif /* !DJGPP */
-
         /* start the I/O; may drop lock */
         bp->flags |= CM_BUF_READING;
         code = (*cm_buf_opsp->Readp)(bp, cm_data.buf_blockSize, &tcount, NULL);
@@ -1096,8 +1110,8 @@ long buf_Get(struct cm_scache *scp, osi_hyper_t *offsetp, cm_buf_t **bufpp)
     }
     lock_ReleaseWrite(&buf_globalLock);
 
-    osi_Log3(buf_logp, "buf_Get returning bp 0x%p for scp 0x%p, offset 0x%x",
-              bp, scp, offsetp->LowPart);
+    osi_Log4(buf_logp, "buf_Get returning bp 0x%p for scp 0x%p, offset 0x%x:%08x",
+              bp, scp, offsetp->HighPart, offsetp->LowPart);
 #ifdef TESTING
     buf_ValidateBufQueues();
 #endif /* TESTING */
@@ -1158,18 +1172,64 @@ void buf_CleanWait(cm_scache_t * scp, cm_buf_t *bp)
  *
  * The buffer must be locked before calling this routine.
  */
-void buf_SetDirty(cm_buf_t *bp)
+void buf_SetDirty(cm_buf_t *bp, afs_uint32 offset, afs_uint32 length)
 {
     osi_assert(bp->magic == CM_BUF_MAGIC);
     osi_assert(bp->refCount > 0);
-	
-    osi_Log1(buf_logp, "buf_SetDirty 0x%p", bp);
 
-    /* set dirty bit */
-    bp->flags |= CM_BUF_DIRTY;
+    lock_ObtainWrite(&buf_globalLock);
+    if (bp->flags & CM_BUF_DIRTY) {
 
-    /* and turn off EOF flag, since it has associated data now */
-    bp->flags &= ~CM_BUF_EOF;
+	osi_Log1(buf_logp, "buf_SetDirty 0x%p already dirty", bp);
+
+        if (bp->dirty_offset <= offset) {
+            if (bp->dirty_offset + bp->dirty_length >= offset + length) {
+                /* dirty_length remains the same */
+            } else {
+                bp->dirty_length = offset + length - bp->dirty_offset;
+            }
+        } else /* bp->dirty_offset > offset */ {
+            if (bp->dirty_offset + bp->dirty_length >= offset + length) {
+                bp->dirty_length = bp->dirty_offset + bp->dirty_length - offset;
+            } else {
+                bp->dirty_length = length;
+            }
+            bp->dirty_offset = offset;
+        }
+    } else {
+	osi_Log1(buf_logp, "buf_SetDirty 0x%p", bp);
+
+        /* set dirty bit */
+        bp->flags |= CM_BUF_DIRTY;
+
+        /* and turn off EOF flag, since it has associated data now */
+        bp->flags &= ~CM_BUF_EOF;
+
+        bp->dirty_offset = offset;
+        bp->dirty_length = length;
+
+        /* and add to the dirty list.  
+         * we obtain a hold on the buffer for as long as it remains 
+         * in the list.  buffers are only removed from the list by 
+         * the buf_IncrSyncer function regardless of when else the
+         * dirty flag might be cleared.
+         *
+         * This should never happen but just in case there is a bug
+         * elsewhere, never add to the dirty list if the buffer is 
+         * already there.
+         */
+        if (bp->dirtyp == NULL && cm_data.buf_dirtyListEndp != bp) {
+            buf_HoldLocked(bp);
+            if (!cm_data.buf_dirtyListp) {
+                cm_data.buf_dirtyListp = cm_data.buf_dirtyListEndp = bp;
+            } else {
+                cm_data.buf_dirtyListEndp->dirtyp = bp;
+                cm_data.buf_dirtyListEndp = bp;
+            }
+            bp->dirtyp = NULL;
+        }
+    }
+    lock_ReleaseWrite(&buf_globalLock);
 }
 
 /* clean all buffers, reset log pointers and invalidate all buffers.
@@ -1194,13 +1254,13 @@ void buf_SetDirty(cm_buf_t *bp)
  */
 long buf_CleanAndReset(void)
 {
-    long i;
+    afs_uint32 i;
     cm_buf_t *bp;
     cm_req_t req;
 
     lock_ObtainWrite(&buf_globalLock);
     for(i=0; i<cm_data.buf_hashSize; i++) {
-        for(bp = cm_data.buf_hashTablepp[i]; bp; bp = bp->hashp) {
+        for(bp = cm_data.buf_scacheHashTablepp[i]; bp; bp = bp->hashp) {
             if ((bp->flags & CM_BUF_DIRTY) == CM_BUF_DIRTY) {
                 buf_HoldLocked(bp);
                 lock_ReleaseWrite(&buf_globalLock);
@@ -1310,7 +1370,7 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
 
     buf_HoldLocked(bufp);
     lock_ReleaseWrite(&buf_globalLock);
-    for(; bufp; bufp = nbufp) {
+    while (bufp) {
         lock_ObtainMutex(&bufp->mx);
 
         bufEnd.HighPart = 0;
@@ -1350,6 +1410,8 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
             if (LargeIntegerLessThanOrEqualTo(*sizep, bufp->offset)) {
                 /* truncating the entire page */
                 bufp->flags &= ~CM_BUF_DIRTY;
+                bufp->dirty_offset = 0;
+                bufp->dirty_length = 0;
                 bufp->dataVersion = -1;	/* known bad */
                 bufp->dirtyCounter++;
             }
@@ -1387,6 +1449,7 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
 	}
 	buf_ReleaseLocked(bufp);
 	lock_ReleaseWrite(&buf_globalLock);
+	bufp = nbufp;
     }
 
 #ifdef TESTING
@@ -1468,6 +1531,34 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     return code;
 }       
 
+/* Must be called with scp->mx held */
+long buf_ForceDataVersion(cm_scache_t * scp, afs_uint32 fromVersion, afs_uint32 toVersion)
+{
+    cm_buf_t * bp;
+    cm_buf_t * nbp;
+    unsigned int i;
+    int found = 0;
+
+    i = BUF_FILEHASH(&scp->fid);
+
+    lock_ObtainWrite(&buf_globalLock);
+
+    for (bp = cm_data.buf_fileHashTablepp[i]; bp; bp = bp->fileHashp) {
+        if (cm_FidCmp(&bp->fid, &scp->fid) == 0) {
+            if (bp->dataVersion == fromVersion) {
+                bp->dataVersion = toVersion;
+                found = 1;
+            }
+        }
+    }
+    lock_ReleaseWrite(&buf_globalLock);
+
+    if (found)
+        return 0;
+    else
+        return ENOENT;
+}
+
 long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
 {
     long code = 0;
@@ -1559,39 +1650,73 @@ buf_ValidateBufQueues(void)
 }
 #endif /* TESTING */
 
-/* dump the contents of the buf_hashTablepp. */
+/* dump the contents of the buf_scacheHashTablepp. */
 int cm_DumpBufHashTable(FILE *outputFile, char *cookie, int lock)
 {
     int zilch;
     cm_buf_t *bp;
     char output[1024];
-    int i;
+    afs_uint32 i;
   
-    if (cm_data.buf_hashTablepp == NULL)
+    if (cm_data.buf_scacheHashTablepp == NULL)
         return -1;
 
     if (lock)
         lock_ObtainRead(&buf_globalLock);
   
-    StringCbPrintfA(output, sizeof(output), "%s - dumping buf_HashTable - buf_hashSize=%d\n", 
+    StringCbPrintfA(output, sizeof(output), "%s - dumping buf_HashTable - buf_hashSize=%d\r\n", 
                     cookie, cm_data.buf_hashSize);
     WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
   
     for (i = 0; i < cm_data.buf_hashSize; i++)
     {
-        for (bp = cm_data.buf_hashTablepp[i]; bp; bp=bp->hashp) 
+        for (bp = cm_data.buf_scacheHashTablepp[i]; bp; bp=bp->hashp) 
         {
-            if (bp->refCount)
-            {
-                StringCbPrintfA(output, sizeof(output), "vnode=%d, unique=%d), size=%d refCount=%d\n", 
-                        cookie, (void *)bp, i, bp->fid.cell, bp->fid.volume, 
-                        bp->fid.vnode, bp->fid.unique, bp->size, bp->refCount);
-                WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
-            }
+	    StringCbPrintfA(output, sizeof(output), 
+			    "%s bp=0x%08X, hash=%d, fid (cell=%d, volume=%d, "
+			    "vnode=%d, unique=%d), offset=%x:%08x, dv=%d, "
+			    "flags=0x%x, cmFlags=0x%x, refCount=%d\r\n",
+			     cookie, (void *)bp, i, bp->fid.cell, bp->fid.volume, 
+			     bp->fid.vnode, bp->fid.unique, bp->offset.HighPart, 
+			     bp->offset.LowPart, bp->dataVersion, bp->flags, 
+			     bp->cmFlags, bp->refCount);
+	    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
         }
     }
   
-    StringCbPrintfA(output, sizeof(output), "%s - Done dumping buf_HashTable.\n", cookie);
+    StringCbPrintfA(output, sizeof(output), "%s - Done dumping buf_HashTable.\r\n", cookie);
+    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
+
+    StringCbPrintfA(output, sizeof(output), "%s - dumping buf_freeListEndp\r\n", cookie);
+    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
+    for(bp = cm_data.buf_freeListEndp; bp; bp=(cm_buf_t *) osi_QPrev(&bp->q)) {
+	StringCbPrintfA(output, sizeof(output), 
+			 "%s bp=0x%08X, fid (cell=%d, volume=%d, "
+			 "vnode=%d, unique=%d), offset=%x:%08x, dv=%d, "
+			 "flags=0x%x, cmFlags=0x%x, refCount=%d\r\n",
+			 cookie, (void *)bp, bp->fid.cell, bp->fid.volume, 
+			 bp->fid.vnode, bp->fid.unique, bp->offset.HighPart, 
+			 bp->offset.LowPart, bp->dataVersion, bp->flags, 
+			 bp->cmFlags, bp->refCount);
+	WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
+    }
+    StringCbPrintfA(output, sizeof(output), "%s - Done dumping buf_FreeListEndp.\r\n", cookie);
+    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
+
+    StringCbPrintfA(output, sizeof(output), "%s - dumping buf_dirtyListEndp\r\n", cookie);
+    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
+    for(bp = cm_data.buf_dirtyListEndp; bp; bp=(cm_buf_t *) osi_QPrev(&bp->q)) {
+	StringCbPrintfA(output, sizeof(output), 
+			 "%s bp=0x%08X, fid (cell=%d, volume=%d, "
+			 "vnode=%d, unique=%d), offset=%x:%08x, dv=%d, "
+			 "flags=0x%x, cmFlags=0x%x, refCount=%d\r\n",
+			 cookie, (void *)bp, bp->fid.cell, bp->fid.volume, 
+			 bp->fid.vnode, bp->fid.unique, bp->offset.HighPart, 
+			 bp->offset.LowPart, bp->dataVersion, bp->flags, 
+			 bp->cmFlags, bp->refCount);
+	WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
+    }
+    StringCbPrintfA(output, sizeof(output), "%s - Done dumping buf_dirtyListEndp.\r\n", cookie);
     WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
 
     if (lock)
@@ -1625,8 +1750,11 @@ long buf_DirtyBuffersExist(cm_fid_t *fidp)
 {
     cm_buf_t *bp;
     afs_uint32 bcount = 0;
+    afs_uint32 i;
 
-    for (bp = cm_data.buf_allp; bp; bp=bp->allp, bcount++) {
+    i = BUF_FILEHASH(fidp);
+
+    for (bp = cm_data.buf_fileHashTablepp[i]; bp; bp=bp->allp, bcount++) {
 	if (!cm_FidCmp(fidp, &bp->fid) && (bp->flags & CM_BUF_DIRTY))
 	    return 1;
     }
@@ -1646,6 +1774,8 @@ long buf_CleanDirtyBuffers(cm_scache_t *scp)
 	    lock_ObtainMutex(&bp->mx);
 	    bp->cmFlags &= ~CM_BUF_CMSTORING;
 	    bp->flags &= ~CM_BUF_DIRTY;
+            bp->dirty_offset = 0;
+            bp->dirty_length = 0;
 	    bp->flags |= CM_BUF_ERROR;
 	    bp->error = VNOVNODE;
 	    bp->dataVersion = -1; /* bad */
