@@ -119,14 +119,16 @@ static long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
 
     if (!fidp) {
         *serversppp = NULL;
-        return 0;
+        return CM_ERROR_INVAL;
     }
 
     cellp = cm_FindCellByID(fidp->cell);
-    if (!cellp) return CM_ERROR_NOSUCHCELL;
+    if (!cellp) 
+        return CM_ERROR_NOSUCHCELL;
 
-    code = cm_GetVolumeByID(cellp, fidp->volume, userp, reqp, &volp);
-    if (code) return code;
+    code = cm_GetVolumeByID(cellp, fidp->volume, userp, reqp, CM_GETVOL_FLAG_CREATE, &volp);
+    if (code) 
+        return code;
     
     *serversppp = cm_GetVolServers(volp, fidp->volume);
 
@@ -139,12 +141,12 @@ static long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
  * and if we're going to retry, determine whether failover is appropriate,
  * and whether timed backoff is appropriate.
  *
- * If the error code is from cm_Conn() or friends, it will be a CM_ERROR code.
+ * If the error code is from cm_ConnFromFID() or friends, it will be a CM_ERROR code.
  * Otherwise it will be an RPC code.  This may be a UNIX code (e.g. EDQUOT), or
  * it may be an RX code, or it may be a special code (e.g. VNOVOL), or it may
  * be a security code (e.g. RXKADEXPIRED).
  *
- * If the error code is from cm_Conn() or friends, connp will be NULL.
+ * If the error code is from cm_ConnFromFID() or friends, connp will be NULL.
  *
  * For VLDB calls, fidp will be NULL.
  *
@@ -162,6 +164,8 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     cm_serverRef_t *tsrp;
     cm_cell_t  *cellp = NULL;
     cm_ucell_t *ucellp;
+    cm_volume_t * volp = NULL;
+    cm_vol_state_t *statep = NULL;
     int retry = 0;
     int free_svr_list = 0;
     int dead_session;
@@ -210,18 +214,25 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     /* leave 5 seconds margin for sleep */
     timeLeft = HardDeadtimeout - timeUsed;
 
+    /* get a pointer to the cell */
+    if (errorCode) {
+        if (cellp == NULL && serverp)
+            cellp = serverp->cellp;
+        if (cellp == NULL && serversp) {
+            struct cm_serverRef * refp;
+            for ( refp=serversp ; cellp == NULL && refp != NULL; refp=refp->next) {
+                if ( refp->server )
+                    cellp = refp->server->cellp;
+            }
+        } 
+        if (cellp == NULL && fidp) {
+            cellp = cm_FindCellByID(fidp->cell);
+        }
+    }
+
     if (errorCode == CM_ERROR_TIMEDOUT) {
         if (timeLeft > 5 ) {
             thrd_Sleep(3000);
-            if (cellp == NULL && serverp)
-                cellp = serverp->cellp;
-            if (cellp == NULL && serversp) {
-                struct cm_serverRef * refp;
-                for ( refp=serversp ; cellp == NULL && refp != NULL; refp=refp->next) {
-                    if ( refp->server )
-                        cellp = refp->server->cellp;
-                }
-            }
             cm_CheckServers(CM_FLAG_CHECKDOWNSERVERS, cellp);
             retry = 1;
         }
@@ -253,75 +264,105 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     }
 
     else if (errorCode == CM_ERROR_ALLOFFLINE) {
-        if (timeLeft > 7) {
-            osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLOFFLINE.");
-            thrd_Sleep(5000);
-            
-	    if (fidp) {	/* Not a VLDB call */
-		if (!serversp) {
-		    code = cm_GetServerList(fidp, userp, reqp, &serverspp);
-		    if (code == 0) {
-			serversp = *serverspp;
-			free_svr_list = 1;
-		    }
-		}
-		if (serversp) {
-		    lock_ObtainWrite(&cm_serverLock);
-		    for (tsrp = serversp; tsrp; tsrp=tsrp->next)
-			tsrp->status = not_busy;
-		    lock_ReleaseWrite(&cm_serverLock);
-		    if (free_svr_list) {
-			cm_FreeServerList(&serversp);
-			*serverspp = serversp;
-		    }
-		    retry = 1;
-		}
+        osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLOFFLINE.");
+        /* Volume instances marked offline will be restored by the 
+         * background daemon thread as they become available 
+         */
+        if (timeLeft > 7 && fidp) {
+            cm_volume_t *volp;
+            cm_vol_state_t *statep;
 
-                cm_ForceUpdateVolume(fidp, userp, reqp);
-	    } else { /* VLDB call */
-		if (serversp) {
-		    lock_ObtainWrite(&cm_serverLock);
-		    for (tsrp = serversp; tsrp; tsrp=tsrp->next)
-			tsrp->status = not_busy;
-		    lock_ReleaseWrite(&cm_serverLock);
-		    if (free_svr_list) {
-			cm_FreeServerList(&serversp);
-			*serverspp = serversp;
-		    }
-		}
-	    }	
+            thrd_Sleep(5000);
+
+            code = cm_GetVolumeByID(cellp, fidp->volume, userp, reqp, 
+                                    CM_GETVOL_FLAG_NO_LRU_UPDATE, 
+                                    &volp);
+            if (code == 0) {
+                if (fidp->volume == volp->rw.ID)
+                    statep = &volp->rw;
+                else if (fidp->volume == volp->ro.ID)
+                    statep = &volp->ro;
+                else if (fidp->volume == volp->bk.ID)
+                    statep = &volp->bk;
+
+                if (statep->state != vl_offline) {
+                    retry = 1;
+                } else {
+                    if (cm_CheckOfflineVolume(volp, statep->ID))
+                        retry = 1;
+                }
+            
+                cm_PutVolume(volp);
+            }
         }
     }
-
-    /* if all servers are busy, mark them non-busy and start over */
     else if (errorCode == CM_ERROR_ALLBUSY) {
+        /* Volumes that are busy cannot be determined to be non-busy 
+         * without actually attempting to access them.
+         */
 	osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLBUSY.");
         if (timeLeft > 7) {
+
             thrd_Sleep(5000);
-            if (!serversp) {
-                code = cm_GetServerList(fidp, userp, reqp, &serverspp);
+
+            if (fidp) { /* File Server query */
+                code = cm_GetVolumeByID(cellp, fidp->volume, userp, reqp, 
+                                        CM_GETVOL_FLAG_NO_LRU_UPDATE, 
+                                        &volp);
                 if (code == 0) {
-                    serversp = *serverspp;
-                    free_svr_list = 1;
+                    if (fidp->volume == volp->rw.ID)
+                        statep = &volp->rw;
+                    else if (fidp->volume == volp->ro.ID)
+                        statep = &volp->ro;
+                    else if (fidp->volume == volp->bk.ID)
+                        statep = &volp->bk;
+
+                    if (statep->state != vl_offline && statep->state != vl_busy) {
+                        retry = 1;
+                    } else {
+                        if (!serversp) {
+                            code = cm_GetServerList(fidp, userp, reqp, &serverspp);
+                            if (code == 0) {
+                                serversp = *serverspp;
+                                free_svr_list = 1;
+                            }
+                        }
+                        lock_ObtainWrite(&cm_serverLock);
+                        for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
+                            if (tsrp->status == srv_busy) {
+                                tsrp->status = srv_not_busy;
+                            }
+                        }
+                        lock_ReleaseWrite(&cm_serverLock);
+                        if (free_svr_list) {
+                            cm_FreeServerList(&serversp, 0);
+                            *serverspp = serversp;
+                        }
+
+                        cm_UpdateVolumeStatus(volp, statep->ID);
+                        retry = 1;
+                    }
+            
+                    cm_PutVolume(volp);
+                }
+            } else {    /* VL Server query */
+                if (serversp) {
+                    lock_ObtainWrite(&cm_serverLock);
+                    for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
+                        if (tsrp->status == srv_busy) {
+                            tsrp->status = srv_not_busy;
+                        }
+                    }
+                    lock_ReleaseWrite(&cm_serverLock);
+                    retry = 1;
                 }
             }
-            lock_ObtainWrite(&cm_serverLock);
-            for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
-                if (tsrp->status == busy)
-                    tsrp->status = not_busy;
-            }
-            lock_ReleaseWrite(&cm_serverLock);
-            if (free_svr_list) {
-                cm_FreeServerList(&serversp);
-                *serverspp = serversp;
-            }
-            retry = 1;
         }
     }
 
     /* special codes:  VBUSY and VRESTARTING */
     else if (errorCode == VBUSY || errorCode == VRESTARTING) {
-        if (!serversp) {
+        if (!serversp && fidp) {
             code = cm_GetServerList(fidp, userp, reqp, &serverspp);
             if (code == 0) {
                 serversp = *serverspp;
@@ -330,15 +371,33 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
         }
         lock_ObtainWrite(&cm_serverLock);
         for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
-            if (tsrp->server == serverp
-                 && tsrp->status == not_busy) {
-                tsrp->status = busy;
+            if (tsrp->server == serverp && tsrp->status == srv_not_busy) {
+                tsrp->status = srv_busy;
+                if (fidp) { /* File Server query */
+                    code = cm_GetVolumeByID(cellp, fidp->volume, userp, reqp, 
+                                             CM_GETVOL_FLAG_NO_LRU_UPDATE, 
+                                             &volp);
+                    if (code == 0) {
+                        if (fidp->volume == volp->rw.ID)
+                            statep = &volp->rw;
+                        else if (fidp->volume == volp->ro.ID)
+                            statep = &volp->ro;
+                        else if (fidp->volume == volp->bk.ID)
+                            statep = &volp->bk;
+                    }
+            
+                    cm_PutVolume(volp);
+                }
                 break;
             }
         }
         lock_ReleaseWrite(&cm_serverLock);
+        
+        if (statep)
+            cm_UpdateVolumeStatus(volp, statep->ID);
+        
         if (free_svr_list) {
-            cm_FreeServerList(&serversp);
+            cm_FreeServerList(&serversp, 0);
             *serverspp = serversp;
         }
         retry = 1;
@@ -405,7 +464,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 #endif
 
         /* Mark server offline for this volume */
-        if (!serversp) {
+        if (!serversp && fidp) {
             code = cm_GetServerList(fidp, userp, reqp, &serverspp);
             if (code == 0) {
                 serversp = *serverspp;
@@ -413,11 +472,19 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
             }
         }
         for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
-            if (tsrp->server == serverp)
-                tsrp->status = offline;
+            if (tsrp->server == serverp) {
+                /* REDIRECT */
+                if (errorCode == VNOVOL || errorCode == VMOVED) {
+                    tsrp->status = srv_deleted;
+                    if (fidp) {
+                        cm_ForceUpdateVolume(fidp, userp, reqp);
+                    }
+                } else 
+                    tsrp->status = srv_offline;
+            }
         }   
         if (free_svr_list) {
-            cm_FreeServerList(&serversp);
+            cm_FreeServerList(&serversp, 0);
             *serverspp = serversp;
         }
         if ( timeLeft > 2 )
@@ -484,9 +551,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     else if (errorCode >= -64 && errorCode < 0) {
         /* mark server as down */
         lock_ObtainMutex(&serverp->mx);
-	if (reqp->flags & CM_REQ_NEW_CONN_FORCED)
+	if (reqp->flags & CM_REQ_NEW_CONN_FORCED) {
 	    serverp->flags |= CM_SERVERFLAG_DOWN;
-	else {
+            serverp->downTime = osi_Time();
+        } else {
 	    reqp->flags |= CM_REQ_NEW_CONN_FORCED;
 	    forcing_new = 1;
 	}
@@ -641,13 +709,13 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
     struct timeval now;
 #endif /* DJGPP */        
 
+    *connpp = NULL;
+
     if (serversp == NULL) {
 	osi_Log1(afsd_logp, "cm_ConnByMServers returning 0x%x", CM_ERROR_ALLDOWN);
 	return CM_ERROR_ALLDOWN;
     }
 
-    *connpp = NULL;
-
 #ifndef DJGPP
     timeUsed = (GetTickCount() - reqp->startTime) / 1000;
 #else
@@ -666,10 +734,12 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
         lock_ReleaseWrite(&cm_serverLock);
         if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
 	    allDown = 0;
-            if (tsrp->status == busy) {
+            if (tsrp->status == srv_deleted) {
+                /* skip this entry.  no longer valid. */;
+            } else if (tsrp->status == srv_busy) {
 		allOffline = 0;
                 someBusy = 1;
-            } else if (tsrp->status == offline) {
+            } else if (tsrp->status == srv_offline) {
 		allBusy = 0;
 		someOffline = 1;
             } else {
@@ -798,6 +868,8 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, cm_conn_t **connpp)
     cm_conn_t *tcp;
     cm_ucell_t *ucellp;
 
+    *connpp = NULL;
+
     lock_ObtainMutex(&userp->mx);
     lock_ObtainWrite(&cm_connLock);
     for (tcp = serverp->connsp; tcp; tcp=tcp->nextp) {
@@ -870,10 +942,10 @@ long cm_ServerAvailable(struct cm_fid *fidp, struct cm_user *userp)
         cm_GetServerNoLock(tsp);
         if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
 	    allDown = 0;
-            if (tsrp->status == busy) {
+            if (tsrp->status == srv_busy) {
 		allOffline = 0;
                 someBusy = 1;
-            } else if (tsrp->status == offline) {
+            } else if (tsrp->status == srv_offline) {
 		allBusy = 0;
 		someOffline = 1;
             } else {
@@ -884,7 +956,7 @@ long cm_ServerAvailable(struct cm_fid *fidp, struct cm_user *userp)
         cm_PutServerNoLock(tsp);
     }   
     lock_ReleaseWrite(&cm_serverLock);
-    cm_FreeServerList(serverspp);
+    cm_FreeServerList(serverspp, 0);
 
     if (allDown)
 	return 0;
@@ -896,24 +968,46 @@ long cm_ServerAvailable(struct cm_fid *fidp, struct cm_user *userp)
 	return 1;
 }
 
-long cm_Conn(struct cm_fid *fidp, struct cm_user *userp, cm_req_t *reqp,
-             cm_conn_t **connpp)
+/* 
+ * The returned cm_conn_t ** object is released in the subsequent call
+ * to cm_Analyze().  
+ */
+long cm_ConnFromFID(struct cm_fid *fidp, struct cm_user *userp, cm_req_t *reqp,
+                    cm_conn_t **connpp)
 {
     long code;
     cm_serverRef_t **serverspp;
 
+    *connpp = NULL;
+
     code = cm_GetServerList(fidp, userp, reqp, &serverspp);
     if (code) {
-        *connpp = NULL;
         return code;
     }
 
     code = cm_ConnByMServers(*serverspp, userp, reqp, connpp);
-    cm_FreeServerList(serverspp);
+    cm_FreeServerList(serverspp, 0);
     return code;
 }
 
-extern struct rx_connection * 
+
+long cm_ConnFromVolume(struct cm_volume *volp, unsigned long volid, struct cm_user *userp, cm_req_t *reqp,
+                       cm_conn_t **connpp)
+{
+    long code;
+    cm_serverRef_t **serverspp;
+
+    *connpp = NULL;
+
+    serverspp = cm_GetVolServers(volp, volid);
+
+    code = cm_ConnByMServers(*serverspp, userp, reqp, connpp);
+    cm_FreeServerList(serverspp, 0);
+    return code;
+}
+
+
+extern struct rx_connection *
 cm_GetRxConn(cm_conn_t *connp)
 {
     struct rx_connection * rxconn;
