@@ -125,10 +125,10 @@ BOOL CALLBACK Columns_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
       return TRUE;
 
    if (msg == WM_INITDIALOG)
-      SetWindowLong (hDlg, DWL_USER, ((LPPROPSHEETPAGE)lp)->lParam);
+      SetWindowLongPtr (hDlg, DWLP_USER, ((LPPROPSHEETPAGE)lp)->lParam);
 
    ShowColumnsParams *pscp;
-   pscp = (ShowColumnsParams *)GetWindowLong (hDlg, DWL_USER);
+   pscp = (ShowColumnsParams *)GetWindowLongPtr (hDlg, DWLP_USER);
 
    switch (msg)
       {
@@ -277,7 +277,7 @@ void Columns_OnSelShown (HWND hDlg)
       }
    else
       {
-      int iAvail = LB_GetData (hList, ii);
+      UINT_PTR iAvail = LB_GetData (hList, ii);
       EnableWindow (GetDlgItem (hDlg, IDC_COL_DELETE), (iAvail != 0));
 
       int ci = (int)SendMessage (hList, LB_GETCOUNT, 0, 0);
