@@ -183,6 +183,7 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
 {
     static char lastcell[MAXCELLCHARS+1] = { 0 };
     static char confname[512] = { 0 };
+    char username_copy[BUFSIZ];
     afs_int32 viceId;			/* AFS uid of user */
 #ifdef ALLOW_REGISTER
     afs_int32 id;
@@ -218,16 +219,6 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
             printf("Id %d\n", viceId);
     }       
 
-    /*
-     * This is a crock, but it is Transarc's crock, so
-     * we have to play along in order to get the
-     * functionality.  The way the afs id is stored is
-     * as a string in the username field of the token.
-     * Contrary to what you may think by looking at
-     * the code for tokens, this hack (AFS ID %d) will
-     * not work if you change %d to something else.
-     */
-
     /*
      * This code is taken from cklog -- it lets people
      * automatically register with the ptserver in foreign cells
@@ -247,16 +238,25 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
             }
 #ifdef ALLOW_REGISTER
         } else if (strcmp(realm_of_user, realm_of_cell) != 0) {
+            int i;
             if (dflag) {
                 printf("doing first-time registration of %s "
                         "at %s\n", username, cell_to_use);
             }
             id = 0;
             strncpy(aclient->name, username, MAXKTCNAMELEN - 1);
+            aclient->name[MAXKTCNAMELEN - 1] = '\0';
             strcpy(aclient->instance, "");
-            strncpy(aclient->cell, c->realm, MAXKTCREALMLEN - 1);
+            strncpy(aclient->cell, cell_to_use, MAXKTCREALMLEN - 1);
+            aclient->cell[MAXKTCREALMLEN - 1] = '\0';
+
+            for ( i=0; aclient->cell[i]; i++ ) {
+                if ( islower(aclient->cell[i]) )
+                    aclient->cell[i] = toupper(aclient->cell[i]);
+            }
+
             if ((*status = ktc_SetToken(aserver, atoken, aclient, 0))) {
-                printf("%s: unable to obtain tokens for cell %s "
+                printf("%s: unable to set tokens for cell %s "
                         "(status: %d).\n", progname, cell_to_use, status);
                 *status = AKLOG_TOKEN;
                 return ;
@@ -274,15 +274,22 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
                 return;
             }
 
-            if ((*status = pr_CreateUser(username, &id))) {
+            /* copy the name because pr_CreateUser lowercases the realm */
+            strncpy(username_copy, username, BUFSIZ);
+
+            *status = pr_CreateUser(username, &id);
+
+            /* and restore the name to the original state */
+            strncpy(username, username_copy, BUFSIZ);
+
+            if (*status) {
                 printf("%s: unable to create remote PTS "
                         "user %s in cell %s (status: %d).\n", progname,
                         username, cell_to_use, *status);
             } else {
-                printf("created cross-cell entry for %s at %s\n",
-                        username, cell_to_use);
+                printf("created cross-cell entry for %s (Id %d) at %s\n",
+                        username, viceId, cell_to_use);
 #ifdef AFS_ID_TO_NAME
-                strncpy(username_copy, username, BUFSIZ);
                 snprintf (username, BUFSIZ, "%s (AFS ID %d)", username_copy, (int) viceId);
 #endif /* AFS_ID_TO_NAME */
             }
@@ -412,75 +419,92 @@ int krb_get_admhst(char *h,char *r, int n);
 
 static char *afs_realm_of_cell(struct afsconf_cell *cellconfig)
 {
-	char krbhst[MAX_HSTNM];
-	static char krbrlm[REALM_SZ+1];
+    char krbhst[MAX_HSTNM];
+    static char krbrlm[REALM_SZ+1];
 
-	if (!cellconfig)
-		return 0;
+    if (!cellconfig)
+	return 0;
 
-	strcpy(krbrlm, (char *) krb_realmofhost(cellconfig->hostName[0]));
+    strcpy(krbrlm, (char *) krb_realmofhost(cellconfig->hostName[0]));
 
-	if (krb_get_admhst(krbhst, krbrlm, 1) != KSUCCESS)
+    if (krb_get_admhst(krbhst, krbrlm, 1) != KSUCCESS)
+    {
+	char *s = krbrlm;
+	char *t = cellconfig->name;
+	int c;
+
+	while (c = *t++)
 	{
-		char *s = krbrlm;
-		char *t = cellconfig->name;
-		int c;
-
-		while (c = *t++)
-		{
-			if (islower(c))
-				c = toupper(c);
-			*s++ = c;
-		}
-		*s++ = 0;
+	    if (islower(c))
+		c = toupper(c);
+	    *s++ = c;
 	}
-	return krbrlm;
+	*s++ = 0;
+    }
+    return krbrlm;
 }
 
-static char *afs_realm_of_cell5(krb5_context context, struct afsconf_cell *cellconfig)
+
+/* As of MIT Kerberos 1.6, krb5_get_host_realm() will return the NUL-string 
+ * if there is no domain_realm mapping for the hostname's domain.  This is 
+ * used as a trigger indicating that referrals should be used within the
+ * krb5_get_credentials() call.  However, if the KDC does not support referrals
+ * that will result in a KRB5_ERR_HOST_REALM_UNKNOWN error and we will have
+ * to manually fallback to mapping the domain of the host as a realm name.
+ * Hence, the new fallback parameter.
+ */
+static char *afs_realm_of_cell5(krb5_context context, struct afsconf_cell *cellconfig, int fallback)
 {
-	char ** krbrlms = 0;
-	static char krbrlm[REALM_SZ+1];
-	krb5_error_code status;
+    char ** krbrlms = 0;
+    static char krbrlm[REALM_SZ+1];
+    krb5_error_code status;
 
-	if (!cellconfig)
-		return 0;
+    if (!cellconfig)
+	return 0;
 
-	status = krb5_get_host_realm( context, cellconfig->hostName[0], &krbrlms );
-
-	if (status == 0 && krbrlms && krbrlms[0]) {
-		strcpy(krbrlm, krbrlms[0]);
+    if (fallback) {
+	char * p;
+	p = strchr(cellconfig->hostName[0], '.');
+	if (p++)
+	    strcpy(krbrlm, p);
+	else
+	    strcpy(krbrlm, cellconfig->name);
+	strupr(krbrlm);
     } else {
-		strcpy(krbrlm, cellconfig->name);
-		strupr(krbrlm);
+	status = krb5_get_host_realm( context, cellconfig->hostName[0], &krbrlms );
+	if (status == 0 && krbrlms && krbrlms[0]) {
+	    strcpy(krbrlm, krbrlms[0]);
+	} else {
+	    strcpy(krbrlm, cellconfig->name);
+	    strupr(krbrlm);
 	}
 
 	if (krbrlms)
-		krb5_free_host_realm( context, krbrlms );
-
-	return krbrlm;
-}
+	    krb5_free_host_realm( context, krbrlms );
+    }
+    return krbrlm;
+}	
 
 static char *copy_cellinfo(cellinfo_t *cellinfo)
 {
-	cellinfo_t *new_cellinfo;
+    cellinfo_t *new_cellinfo;
 
-	if (new_cellinfo = (cellinfo_t *)malloc(sizeof(cellinfo_t)))
-		memcpy(new_cellinfo, cellinfo, sizeof(cellinfo_t));
+    if (new_cellinfo = (cellinfo_t *)malloc(sizeof(cellinfo_t)))
+	memcpy(new_cellinfo, cellinfo, sizeof(cellinfo_t));
 
-	return ((char *)new_cellinfo);
+    return ((char *)new_cellinfo);
 }
 
 
 static char *copy_string(char *string)
 {
-	char *new_string;
+    char *new_string;
 
-	if (new_string = (char *)calloc(strlen(string) + 1, sizeof(char)))
-		(void) strcpy(new_string, string);
+    if (new_string = (char *)calloc(strlen(string) + 1, sizeof(char)))
+	(void) strcpy(new_string, string);
 
-	return (new_string);
-}
+    return (new_string);
+}	
 
 
 static int get_cellconfig(char *cell, struct afsconf_cell *cellconfig,
@@ -538,10 +562,10 @@ static int get_v5_user_realm(krb5_context context,char *realm)
 static int auth_to_cell(krb5_context context, char *cell, char *realm)
 {
     int status = AKLOG_SUCCESS;
-    char username[BUFSIZ];	/* To hold client username structure */
+    char username[BUFSIZ];	  /* To hold client username structure */
 
-    char name[ANAME_SZ];		/* Name of afs key */
-    char instance[INST_SZ];	/* Instance of afs key */
+    char name[ANAME_SZ];	  /* Name of afs key */
+    char instance[INST_SZ];	  /* Instance of afs key */
     char realm_of_user[REALM_SZ]; /* Kerberos realm of user */
     char realm_of_cell[REALM_SZ]; /* Kerberos realm of cell */
     char local_cell[MAXCELLCHARS+1];
@@ -552,7 +576,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
     struct ktc_principal aserver;
     struct ktc_principal aclient;
     struct ktc_token atoken, btoken;
-
+    int i;
 
     /* try to avoid an expensive call to get_cellconfig */
     if (cell && ll_string_check(&authedcells, cell))
@@ -591,14 +615,6 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
     if (dflag)
         printf("Authenticating to cell %s.\n", cell_to_use);
 
-    if (realm && realm[0])
-        strcpy(realm_of_cell, realm);
-    else
-        strcpy(realm_of_cell,
-                (usev5)? 
-                afs_realm_of_cell5(context, &ak_cellconfig) : 
-                afs_realm_of_cell(&ak_cellconfig));
-
     /* We use the afs.<cellname> convention here... */
     strcpy(name, AFSKEY);
     strncpy(instance, cell_to_use, sizeof(instance));
@@ -612,6 +628,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
     if (usev5) 
     { /* using krb5 */
         int retry = 1;
+	int realm_fallback = 0;
 
         if ( strchr(name,'.') != NULL ) {
             fprintf(stderr, "%s: Can't support principal names including a dot.\n",
@@ -620,24 +637,43 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
         }
 
       try_v5:
-        if (dflag)
+	if (realm && realm[0])
+	    strcpy(realm_of_cell, realm);
+	else
+	    strcpy(realm_of_cell,
+		    afs_realm_of_cell5(context, &ak_cellconfig, realm_fallback));
+
+	if (dflag)
             printf("Getting v5 tickets: %s/%s@%s\n", name, instance, realm_of_cell);
         status = get_v5cred(context, name, instance, realm_of_cell, 
                             use524 ? &c : NULL, &v5cred);
-        if (status == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN) {
+	if (status == KRB5_ERR_HOST_REALM_UNKNOWN) {
+	    realm_fallback = 1;
+	    goto try_v5;
+	} else if (status == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN) {
+	    if (!realm_of_cell[0]) {
+		realm_fallback = 1;
+		goto try_v5;
+	    }
             if (dflag)
                 printf("Getting v5 tickets: %s@%s\n", name, realm_of_cell);
             status = get_v5cred(context, name, "", realm_of_cell, 
                                 use524 ? &c : NULL, &v5cred);
-        }
+	}
         if ( status == KRB5KRB_AP_ERR_MSG_TYPE && retry ) {
             retry = 0;
+	    realm_fallback = 0;
             goto try_v5;
         }       
     }       
     else 
     {
-        /*
+	if (realm && realm[0])
+	    strcpy(realm_of_cell, realm);
+	else
+	    strcpy(realm_of_cell, afs_realm_of_cell(&ak_cellconfig));
+
+	/*
          * Try to obtain AFS tickets.  Because there are two valid service
          * names, we will try both, but trying the more specific first.
          *
@@ -777,6 +813,11 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
     } else
 	strncpy(aclient.cell, c.realm, MAXKTCREALMLEN - 1);
 
+    for ( i=0; aclient.cell[i]; i++ ) {
+        if ( islower(aclient.cell[i]) )
+            aclient.cell[i] = toupper(aclient.cell[i]);
+    }
+
     if (dflag)
         printf("Getting tokens.\n");
     if (status = ktc_SetToken(&aserver, &atoken, &aclient, 0))
