@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/budb/struct_ops.c,v 1.6 2003/07/15 23:14:48 shadow Exp $");
+    ("$Header: /cvs/openafs/src/budb/struct_ops.c,v 1.8 2006/04/04 20:51:16 shadow Exp $");
 
 #include <stdio.h>
 #ifdef AFS_NT40_ENV
@@ -36,21 +36,22 @@ RCSID
  * ----------------------------------
  */
 
-printDbHeader(ptr)
-     struct DbHeader *ptr;
+void
+printDbHeader(struct DbHeader *ptr)
 {
+    time_t created = ptr->created;
     printf("version = %d\n", ptr->dbversion);
-    printf("created = %s", ctime((time_t *) & ptr->created));
+    printf("created = %s", ctime(&created));
     printf("cell = %s\n", ptr->cell);
     printf("lastDumpId = %u\n", ptr->lastDumpId);
     printf("lastInstanceId = %d\n", ptr->lastInstanceId);
     printf("lastTapeId = %d\n", ptr->lastTapeId);
 }
 
-printDump(fid, dptr)
-     FILE *fid;
-     struct dump *dptr;
+void
+printDump(FILE *fid, struct dump *dptr)
 {
+    time_t created = dptr->created;
     fprintf(fid, "id = %u\n", dptr->id);
     fprintf(fid, "idHashChain = %d\n", dptr->idHashChain);
     fprintf(fid, "name = %s\n", dptr->dumpName);
@@ -59,7 +60,7 @@ printDump(fid, dptr)
     fprintf(fid, "nameHashChain = %d\n", dptr->nameHashChain);
     fprintf(fid, "flags = 0x%x\n", dptr->flags);
     fprintf(fid, "parent = %u\n", dptr->parent);
-    fprintf(fid, "created = %s", ctime((time_t *) & dptr->created));
+    fprintf(fid, "created = %s", ctime(&created));
     fprintf(fid, "nVolumes = %d\n", dptr->nVolumes);
     /* printTapeSet(&dptr->tapes); */
     fprintf(fid, "firstTape = %d\n", dptr->firstTape);
@@ -67,9 +68,10 @@ printDump(fid, dptr)
 
 }
 
-printDumpEntry(deptr)
-     struct budb_dumpEntry *deptr;
+void 
+printDumpEntry(struct budb_dumpEntry *deptr)
 {
+    time_t created = deptr->created;
     printf("id = %u\n", deptr->id);
     printf("Initial id = %u\n", deptr->initialDumpID);
     printf("Appended id = %u\n", deptr->appendedDumpID);
@@ -94,7 +96,7 @@ printDumpEntry(deptr)
     printf("volumeSet = %s\n", deptr->volumeSetName);
     printf("dump path = %s\n", deptr->dumpPath);
     printf("name = %s\n", deptr->name);
-    printf("created = %s", ctime((time_t *) & deptr->created));
+    printf("created = %s", ctime(&created));
     printf("nVolumes = %d\n", deptr->nVolumes);
 
     printTapeSet(&deptr->tapes, (deptr->flags & BUDB_DUMP_XBSA_NSS));
@@ -122,6 +124,7 @@ printHashTable(fid, htptr)
 /* printMemoryHashTable
  *	print the hash table structure, i.e. the header structure.
  */
+int
 printMemoryHashTable(fid, mhtptr)
      FILE *fid;
      struct memoryHashTable *mhtptr;
@@ -131,32 +134,37 @@ printMemoryHashTable(fid, mhtptr)
     fprintf(fid, "progress = %d\n", mhtptr->progress);
     fprintf(fid, "size = %d\n", mhtptr->size);
     fprintf(fid, "oldsize = %d\n", mhtptr->oldSize);
+    return 0;
 }
 
+int
 printPrincipal(ptr)
      struct ktc_principal *ptr;
 {
     printf("name = %s\n", ptr->name);
     printf("instance = %s\n", ptr->instance);
     printf("cell = %s\n", ptr->cell);
+    return 0;
 }
 
+int
 printStructDumpHeader(ptr)
      struct structDumpHeader *ptr;
 {
     printf("type = %d\n", ptr->type);
     printf("structure version = %d\n", ptr->structversion);
     printf("size = %d bytes\n", ptr->size);
+    return 0;
 }
 
-printTape(fid, tptr)
-     FILE *fid;
-     struct tape *tptr;
+int
+printTape(FILE *fid, struct tape *tptr)
 {
+    time_t written = tptr->written;
     fprintf(fid, "name = %s\n", tptr->name);
     fprintf(fid, "nameHashChain = %d\n", tptr->nameHashChain);
     fprintf(fid, "flags = 0x%x\n", tptr->flags);
-    fprintf(fid, "written = %s", ctime((time_t *) & tptr->written));
+    fprintf(fid, "written = %s", ctime(&written));
     fprintf(fid, "nMBytes = %d\n", tptr->nMBytes);
     fprintf(fid, "nBytes = %d\n", tptr->nBytes);
     fprintf(fid, "nFiles = %d\n", tptr->nFiles);
@@ -167,11 +175,15 @@ printTape(fid, tptr)
     fprintf(fid, "firstVol = %d\n", tptr->firstVol);
     fprintf(fid, "labelPos = %d\n", tptr->labelpos);
     fprintf(fid, "useCount = %d\n", tptr->useCount);
+    return 0;
 }
 
-printTapeEntry(teptr)
-     struct budb_tapeEntry *teptr;
+int
+printTapeEntry(struct budb_tapeEntry *teptr)
 {
+    time_t written = teptr->written;
+    time_t expires = teptr->expires;
+
     printf("name = %s\n", teptr->name);
     printf("flags = 0x%x", teptr->flags);
     if (teptr->flags & BUDB_TAPE_TAPEERROR)
@@ -187,8 +199,8 @@ printTapeEntry(teptr)
     if (teptr->flags & BUDB_TAPE_WRITTEN)
 	printf(": Successful");
     printf("\n");
-    printf("written = %s", ctime((time_t *) & teptr->written));
-    printf("expires = %s", cTIME((time_t *) & teptr->expires));
+    printf("written = %s", ctime(&written));
+    printf("expires = %s", cTIME(&expires));
     printf("kBytes Tape Used = %u\n", teptr->useKBytes);
     printf("nMBytes Data = %d\n", teptr->nMBytes);
     printf("nBytes  Data = %d\n", teptr->nBytes);
@@ -198,8 +210,10 @@ printTapeEntry(teptr)
     printf("labelPos = %d\n", teptr->labelpos);
     printf("useCount = %d\n", teptr->useCount);
     printf("dump = %d\n", teptr->dump);
+    return 0;
 }
 
+int
 printTapeSet(tsptr, nss)
      struct budb_tapeSet *tsptr;
      afs_int32 nss;		/* is the tapeserver name an accurate name */
@@ -212,11 +226,13 @@ printTapeSet(tsptr, nss)
 /*  printf("a  = %d\n",tsptr->a ); */
 /*  printf("b = %d\n",tsptr->b);   */
     printf("Start Tape Seq = %d\n", tsptr->b);
+    return 0;
 }
 
-printVolumeEntry(veptr)
-     struct budb_volumeEntry *veptr;
+int
+printVolumeEntry(struct budb_volumeEntry *veptr)
 {
+    time_t clone = veptr->clone;
     printf("name = %s\n", veptr->name);
     printf("flags = 0x%x", veptr->flags);
     if (veptr->flags & BUDB_VOL_TAPEERROR)
@@ -238,32 +254,36 @@ printVolumeEntry(veptr)
     printf("tapeSeq = %d\n", veptr->tapeSeq);
 
     printf("position = %d\n", veptr->position);
-    printf("clone = %s", ctime((time_t *) & veptr->clone));
+    printf("clone = %s", ctime(&clone));
     printf("startByte = %d\n", veptr->startByte);
     printf("nBytes = %d\n", veptr->nBytes);
     printf("seq = %d\n", veptr->seq);
 
     printf("dump = %d\n", veptr->dump);
     printf("tape = %s\n", veptr->tape);
+    return 0;
 }
 
-printVolFragment(fid, vfptr)
-     FILE *fid;
-     struct volFragment *vfptr;
+int
+printVolFragment(FILE *fid, struct volFragment *vfptr)
 {
+    time_t clone = vfptr->clone;
+    time_t incTime = vfptr->incTime;
     fprintf(fid, "vol = %d\n", vfptr->vol);
     fprintf(fid, "sameNameChain = %d\n", vfptr->sameNameChain);
     fprintf(fid, "tape = %d\n", vfptr->tape);
     fprintf(fid, "sameTapeChain = %d\n", vfptr->sameTapeChain);
     fprintf(fid, "position = %d\n", vfptr->position);
-    fprintf(fid, "clone = %s", ctime((time_t *) & vfptr->clone));
-    fprintf(fid, "incTime = %s", ctime((time_t *) & vfptr->incTime));
+    fprintf(fid, "clone = %s", ctime(&clone));
+    fprintf(fid, "incTime = %s", ctime(&incTime));
     fprintf(fid, "startByte = %d\n", vfptr->startByte);
     fprintf(fid, "nBytes = %d\n", vfptr->nBytes);
     fprintf(fid, "flags = %d\n", vfptr->flags);
     fprintf(fid, "sequence = %d\n", vfptr->sequence);
+    return 0;
 }
 
+int
 printVolInfo(fid, viptr)
      FILE *fid;
      struct volInfo *viptr;
@@ -278,6 +298,7 @@ printVolInfo(fid, viptr)
     fprintf(fid, "sameNameChain = %d\n", viptr->sameNameChain);
     fprintf(fid, "firstFragment = %d\n", viptr->firstFragment);
     fprintf(fid, "nFrags = %d\n", viptr->nFrags);
+    return 0;
 }
 
 
@@ -401,6 +422,7 @@ principal_hton(hostptr, netptr)
     strcpy(netptr->name, hostptr->name);
     strcpy(netptr->instance, hostptr->instance);
     strcpy(netptr->cell, hostptr->cell);
+    return 0;
 }
 
 principal_ntoh(netptr, hostptr)
@@ -409,6 +431,7 @@ principal_ntoh(netptr, hostptr)
     strcpy(hostptr->name, netptr->name);
     strcpy(hostptr->instance, netptr->instance);
     strcpy(hostptr->cell, netptr->cell);
+    return 0;
 }
 
 structDumpHeader_hton(hostPtr, netPtr)
@@ -445,6 +468,7 @@ tapeEntry_ntoh(netptr, hostptr)
     hostptr->dump = ntohl(netptr->dump);
 }
 
+int
 tapeSet_hton(hostptr, netptr)
      struct budb_tapeSet *hostptr, *netptr;
 {
@@ -454,8 +478,10 @@ tapeSet_hton(hostptr, netptr)
     netptr->maxTapes = htonl(hostptr->maxTapes);
     netptr->a = htonl(hostptr->a);
     netptr->b = htonl(hostptr->b);
+    return 0;
 }
 
+int
 tapeSet_ntoh(netptr, hostptr)
      struct budb_tapeSet *netptr, *hostptr;
 {
@@ -465,6 +491,7 @@ tapeSet_ntoh(netptr, hostptr)
     hostptr->maxTapes = ntohl(netptr->maxTapes);
     hostptr->a = ntohl(netptr->a);
     hostptr->b = ntohl(netptr->b);
+    return 0;
 }
 
 textBlock_hton(hostptr, netptr)
